/*
 * Copyright (C) 2017-2020 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STRINGS_HPP
#define STRINGS_HPP

#include "compiler.hpp"

#include <string>
#include <cctype>
#include <cstring>
#include <vector>

inline bool begins_with(const std::string& s, const std::string& b) {
    return s.find(b) == 0;
}

inline bool ends_with(const std::string& s, const std::string& e) {
    if(s.size() < e.size())
        return false;
    std::string::size_type position = s.rfind(e);
    if (position == std::string::npos)
        return false;
    return s.substr(position) == e;
}

inline auto upper_case(const std::string& s) {
    std::string out;
    for(size_t pos = 0; pos < s.size(); ++pos) {
        out += toupper(s[pos]);
    }
    return out;
}

inline auto lower_case(const std::string& s) {
    std::string out;
    for(size_t pos = 0; pos < s.size(); ++pos) {
        out += tolower(s[pos]);
    }
    return out;
}

inline auto strip(const std::string& str) {
    size_t first = str.find_first_not_of(' ');
    size_t last = str.find_last_not_of(' ');
    return str.substr(first, (last-first+1));
}

inline bool eq(const char *p1, const char *p2) {
    if(!p1 && !p2)
        return true;
    if(!p1 || !p2)
        return false;
    return !strcmp(p1, p2);
}

inline bool eq(const char *p1, const char *p2, size_t len) {
    if(!p1 && !p2)
        return true;
    if(!p1 || !p2)
        return false;
    return !strncmp(p1, p2, len);
}

const char *get_env(const char *id, size_t max);
size_t u8verify(const char *cp, size_t max);
std::vector<std::string> split(const std::string& str, const std::string& delim = " ");

/*!
 * Common string functions and extensions to std::string.
 * \file strings.hpp
 */

#endif
